#photoo from kaswy
#Website http://kaswy.free.fr
#
#Edited by Thomas Hansen and Juan Pablo Hourcade for Open Autism Software
# http://www.openautismsoftware.org
#
#This software is provided under the GNU General Public License, version 3
#See the license at http://www.gnu.org/licenses/gpl.html

from __future__ import with_statement
import sys
import pygame
from math import sqrt
from pymt import *
from OpenGL.GL import *
from OpenGL.GLU import *
import time
res = 15 #resolution of the grid used to deform the picture
Data = []
vertexImpact = {}
from glob import glob

cont = MTWidget()


#load all the jpg files in the directory and save them in a cache
files = glob("../../../photos/*.jpg")
image_cache = {}
for i in range(len(files)):
        f = files[i]
        image_cache[f] = Image(f)

#this presents the menu of images
class ImageMenu(MTScatterPlane):
    def __init__(self):
        super(ImageMenu, self).__init__()
        do_rotation = False
        do_translation = False
        do_scale = False

    def on_touch_move(self, touch):
        return

#this is the widget that handles the deformation of the picture
class GLWindow(MTWidget):
	def __init__(self):
		super(GLWindow, self).__init__()
		self.draw_color = (1.0,1.0,1.0)
		self.touch_position = {}
		self.rot_x, self.rot_y = 0.0, 0.0
		self.need_init = True
		self.restart = False

		with fbo:
		    set_color(1,1,1,1)
		    glClear(GL_COLOR_BUFFER_BIT)
		    with gx_matrix:
			    glMatrixMode(GL_MODELVIEW)
			    glLoadIdentity()
			    drawTexturedRectangle(image.texture, size=fbo.size)

    #draws the mesh with the image as a texture, as it has been deformed
	#the mesh and vertex coordinates are originally the same, so when the mesh coordinates are moved, the corresponding texture coordinates still point to the same place in the image
	def on_draw(self):
		if self.need_init:
			self.pick_file()
			return

		with gx_matrix:
			glMatrixMode(GL_MODELVIEW)
			glLoadIdentity()
			glScaled(getWindow().width, getWindow().height, 1)
			set_color(1,1,1)
			with gx_texture(fbo.texture):
				Vertex = 0

				for y in range(res-1):
				    glBegin(GL_QUAD_STRIP)
				    for x in range(res):
					xyz = Data[Vertex]
					glTexCoord2f(float(x)/res,float(y)/res);
					glVertex2f(xyz[0]/(res-1),xyz[1]/(res-1))
					Vertex += 1
					xyz = Data[Vertex]
					glTexCoord2f(float(x)/res,float(y+1)/res);
					glVertex2f(xyz[0]/(res-1),xyz[1]/(res-1))
					Vertex += 1
				    glEnd()


	def on_touch_down(self, touch):
		if self.need_init:
			return
		if touch.device =='wm_pen':
			return
		x,y = touch.pos
		vertexImpact[touch.id]=[(x,y)]
		for xy in Data:
			impact = ((sqrt(abs((xy[0]*dx-x)**2 + (xy[1]*dy-y)**2))/diag)**1.5)*60000
			vertexImpact[touch.id].append(impact)

	#when a touch is moved the mesh nodes in the area around the touch are moved in the same direction with an expoential fator (further away from the touch moves less than right under the touch)
	def on_touch_move(self, touch):
		if self.need_init:
			return
		if touch.device =='wm_pen':
			return
		x,y = touch.pos
		startpos = vertexImpact[touch.id][0]
		deplx=x-startpos[0]
		deply=y-startpos[1]
		vertex = 0
		for impact in vertexImpact[touch.id][1:]:
			if impact < 200: impact = 200.0
			Data[vertex][0]+= deplx/impact
			Data[vertex][1]+= deply/impact
			vertex+=1
		vertexImpact[touch.id][0]=(x,y)
		imp=1
		for xy in Data:
			impact = ((sqrt(abs((xy[0]*dx-x)**2 + (xy[1]*dy-y)**2))/diag)**1.5)*40000
			if vertexImpact[touch.id][imp] > impact :vertexImpact[touch.id][imp] = impact
			imp+=1

	def pick_file(self):
        #add a plane above and add image buttons for every image loaded at the beginning and reinitialize the mesh when one is selected
		picker = ImageMenu()
		files = glob("../../../photos/*.jpg")
		for i in range(len(files)):
			f = files[i]

			def click(f, touch):
				cont.children.remove(picker)
				self.load_image(f)
				cont.add_widget(self)
				self.need_init = False

			btn_x = MTButton(label=" ",style={'bg-image':image_cache[f]}, size=(170,170), pos=(15+(i/4)*190, 15+(i%4)*190))
			btn_x.push_handlers(on_release=curry(click, f))
			picker.add_widget(btn_x)
		cont.remove_widget(self)
		cont.add_widget(picker)

    #loads and image from the cache (draws it into a fbo to be used as texture) and resets the mesh
	def load_image(self, src):
		global image
		print "loading:", src
		image = image_cache[src]

		try:
			with fbo:
			    set_color(1,1,1,1)
			    glClear(GL_COLOR_BUFFER_BIT)
			    with gx_matrix:
				    glMatrixMode(GL_MODELVIEW)
				    glLoadIdentity()
				    drawTexturedRectangle(image.texture, size=fbo.size)
		except:
			print "caught"

		createMesh()
		print "created mesh"

        def on_touch_up(self, touch):
            if self.need_init:
                return
            if vertexImpact.get(touch.id):
                del vertexImpact[touch.id]


#watches for simultaneous touches on bottom left and upper right corners to go back to menu of images
class ExitWidget(MTWidget):

    def __init__(self, **kwargs):
        super(ExitWidget, self).__init__(**kwargs)
        self.goo_widget = kwargs.get('goo_widget')
        self.pen_drawings = kwargs.get('pen_widget')


    def on_touch_down(self,touch):
        if self.goo_widget.need_init:
            return
        bottom_left = False
        top_right = False
        all_touches = getCurrentTouches()
        for t in all_touches:
            if t.x < 100 and t.y < 100:
                bottom_left = True
            if t.x > getWindow().width -100 and t.y > getWindow().height -100:
                top_right = True
        #print bottom_left, top_right, bottom_left and top_right
        if bottom_left and top_right:
            time.sleep(0.2)
            self.goo_widget.need_init = True
            self.pen_drawings.cleanup()
            return True

def createMesh():
	global Data
	Data = []
	for y in xrange(res):
		for x in xrange(res):
			Data.append([1.0*x,1.0*y])
			Data.append([1.0*x,1.0*y+1])

def loadTextures(src = '..\..\..\photos\portrait.jpg'):
		global image, image2, fbo, fbo2

		image = Image(src)
		fbo = Fbo(size =( 512,512))
		fbo2 = Fbo(size =( 512,512))

#this widget is used to draw on top of the image with a pen
class Painter(MTWidget):
        def init(self):
                self.lines = []
                self.line = None

        def on_update(self):
                self.bring_to_front()
        def on_touch_down(self, touch):
                if touch.device =='wm_pen':
                        self.line = list(touch.pos)
                        self.lines.append(self.line)
                        return True
        def on_touch_move(self, touch):
                if touch.device =='wm_pen':
                        self.line.extend(touch.pos)
                        return True
        def on_touch_up(self, touch):
                if touch.device =='wm_pen':
                        self.line.extend(touch.pos)
                        return True

        def draw(self):
                set_color(0,0,0)
                #print "drawing", self.lines
                for line in self.lines:
                        drawLine(line, width=4)
        def cleanup(self):
            self.lines = []
            self.line = None

createMesh()
loadTextures()
w = GLWindow()
w.size = getWindow().size

dx=w.width/(res-1)
dy=w.height/(res-1)
diag=sqrt(w.width**2+w.height**2)

root = MTWidget()

cont.add_widget(w)
root.add_widget(cont)
p=Painter()
root.add_widget(p)

root.add_widget(ExitWidget(goo_widget = w, pen_widget = p))

runTouchApp(root)
sys.exit()
